import 'dart:convert';
import 'dart:typed_data';
import 'package:fis_lib_pdf/override/custom_text.dart' as custom;
import 'package:fis_lib_pdf/ultils/recources_catch.dart';
import 'package:intl/intl.dart';
import 'package:pdf/pdf.dart';
import 'package:pdf/widgets.dart' as pw;
import '../report_helper/report_dic.dart';
import '../report_helper/report_style_helper.dart';

///本实体类实现了对模板树当中的叶子节点信息的对应PDF组件的转换
class PdfLeafElement {
  ///StaticTextInfo 转化为一行pd文本内容
  ///将换行符通过SizeBox做替代
  static pw.Widget getTextPdfWidget(
      StaticTextInfo elementInfo, bool parentIsTable, bool parentIsRTCell) {
    var staticText = elementInfo.element as StaticText;

    var fontSize = staticText.fontSize!;
    var leftItemWithTable =
        staticText.horizontalAlignment == HorizontalLayout.Left &&
            parentIsTable;

    ///table 内元素应该用container 调整其左对齐。
    if (leftItemWithTable) {
      return pw.Container(
        alignment: pw.Alignment.centerLeft,
        child: pw.Text(
          _getCleanText(staticText.text!),
          style: pw.TextStyle(
              lineSpacing: 0,
              font: ResourcesCache.pdfCustomFonts["defaultCN"],
              color: ReportStyleHelper.getPDFColorFromRTColor(
                  staticText.fontColor!),
              fontSize: fontSize),
        ),
      );
    }
    var spaceWidget = _caculateSpecialSpaceToWidget(staticText, fontSize);

    var widget = pw.Wrap(
        crossAxisAlignment: ReportStyleHelper.getWrapCrossAlignment(
            staticText.verticalAlignment!),
        alignment:
            ReportStyleHelper.getWrapAlignment(staticText.horizontalAlignment!),
        children: [
          ...spaceWidget,
        ]);

    return widget;
  }

  static pw.Widget _getStaticTextWithGivenFontSize(
      String text, double fontSizeCaculated, RTColor fontColr) {
    return pw.Text(
      text,
      style: pw.TextStyle(
          lineSpacing: 0,
          font: ResourcesCache.pdfCustomFonts["defaultCN"],
          color: ReportStyleHelper.getPDFColorFromRTColor(fontColr),
          fontSize: fontSizeCaculated),
    );
  }

  static List<pw.Widget> _caculateSpecialSpaceToWidget(
      StaticText textElement, double fontSize) {
    List<pw.Widget> textWidgets = [];
    var text = _getCleanText(textElement.text.toString());
    final containsSpace = text.contains('      ');
    if (containsSpace) {
      for (var i = 0; i < text.length; i++) {
        if (text[i] != ' ') {
          textWidgets.add(
            _getStaticTextWithGivenFontSize(
                text[i], fontSize, textElement.fontColor!),
          );
        } else {
          textWidgets.add(pw.SizedBox(
            width: 5.4,
          ));
        }
      }
    } else {
      textWidgets.add(_getStaticTextWithGivenFontSize(
        text!,
        fontSize,
        textElement.fontColor!,
      ));
    }
    return textWidgets;
  }

  ///将LineInfo转换为具有PDF divider水平线的容器
  static pw.Widget getLinePdfWidget(LineInfo elementInfo) {
    var line = elementInfo.element as Line;
    var lineHeight = line.thickness! + 5;
    var pwmargin = ReportStyleHelper.convertRTMargin(line.margin);

    return pw.Container(
      margin: pwmargin,
      alignment: ReportStyleHelper.getRowMainAlign(line.horizontalAlignment!),
      width: line.width,
      child: pw.Divider(
          thickness: line.thickness,
          height: lineHeight,
          borderStyle: line.borderStyle == null
              ? null
              : ReportStyleHelper.getPDFBorderStyle(line.borderStyle!),
          color: ReportStyleHelper.getPDFColorFromRTColor(line.stroke!)),
    );
  }

  ///将DateTimeInfo转换为具有PDF的单纯文本
  static pw.Widget getDateTimePdfWidget(DateTimeInfo elementInfo) {
    var dateTime = elementInfo.element as DateTimeElement;

    var date = DateTime.parse(elementInfo.text.trim());
    final DateFormat formatter = DateFormat(dateTime.dateTimeFormat);
    var dateStr = formatter.format(date);
    return pw.Text(dateStr,
        style: pw.TextStyle(
            font: ResourcesCache.pdfCustomFonts["defaultCN"],
            color:
                ReportStyleHelper.getPDFColorFromRTColor(dateTime.fontColor!),
            fontSize: dateTime.fontSize));
  }

  ///将InputTextInfo转换为具有PDF的文本组件的文本行
  static pw.Widget getInputTextPdfWidget(
      InputTextInfo elementInfo, bool parentElementHasNoContainer) {
    var inputText = elementInfo.element as InputText;
    var text = elementInfo.text;

    ///在PDF 3.8.1当中，来自业务数据的空格编码有时不能正确解析。
    /// 2024/2/23注释：注释该行后并未发现乱码，修复bug 0017221
    // text = text.replaceAll(" ", "");

    ///在PDF 3.8.1当中,‘\r’ 在默认字体下出现乱码。
    text = text.replaceAll('\r', "");

    text = text.replaceAll(" ", " "); // 将 U+00A0 替换为普通空格，否则会出现“!”乱码

    /// 如果是测量值，需要根据测量值的类型进行格式化
    if (elementInfo.measureTag != null &&
        elementInfo.measureTag!.measureResultType != null &&
        elementInfo.measureTag!.measureResultType == 'Measures') {
      text = getMeasureValue(text);
    }

    final containsChinese = isContainChinese(text);

    if (parentElementHasNoContainer) {
      ///PDF主段落应用Paragraph来承载，因为PDF3.8.1的Text在分页时支持度不好。

      var widget = pw.Container(
        alignment:
            ReportStyleHelper.getRowMainAlign(inputText.horizontalAlignment!),
        child: pw.Container(
          margin: const pw.EdgeInsets.only(bottom: 5.0 * PdfPageFormat.mm),
          child: custom.CustomRichText(
            text: custom.TextSpan(
              text: text,
              style: pw.TextStyle(
                  font: ResourcesCache.pdfCustomFonts["defaultCN"],
                  color: ReportStyleHelper.getPDFColorFromRTColor(
                      inputText.fontColor!),
                  fontSize: inputText.fontSize),
            ),
            textScaleFactor: 1.0,
            disableWordsSplit: containsChinese,
            overflow: pw.TextOverflow.span,
          ),
        ),
      );

      // child: pw.Paragraph(
      //     text: text,
      //     style: pw.TextStyle(
      //         font: ResourcesCache.pdfCustomFonts["defaultCN"],
      //         color: ReportStyleHelper.getPDFColorFromRTColor(
      //             inputText.fontColor!),
      //         fontSize: inputText.fontSize)));

      return widget;
    } else {
      var widget = pw.Container(
          alignment:
              ReportStyleHelper.getRowMainAlign(inputText.horizontalAlignment!),
          child: pw.Text(text,
              style: pw.TextStyle(
                  font: ResourcesCache.pdfCustomFonts["defaultCN"],
                  color: ReportStyleHelper.getPDFColorFromRTColor(
                      inputText.fontColor!),
                  fontSize: inputText.fontSize)));

      return widget;
    }
  }

  static String getMeasureValue(String originStr) {
    //// 获取 originStr 中第一个 '[' 和最后一个 ']' 之间的内容
    int firstIndex = originStr.indexOf('[');
    int lastIndex = originStr.lastIndexOf(']');
    if (firstIndex == -1 || lastIndex == -1) {
      return "";
    }
    if (firstIndex > lastIndex) {
      return "";
    }
    if (firstIndex <= 0) {
      return "";
    }
    try {
      return originStr.substring(firstIndex, lastIndex + 1);
    } catch (e) {
      return "";
    }
  }

  // 是否包含中文字符
  static bool isContainChinese(String str) {
    return str.contains(RegExp(r'[\u4e00-\u9fa5]'));
  }

  ///将SingleSelectedInfo转换为具有PDF的文本组件的文本行
  static pw.Widget getSingleSelectPdfWidget(SingleSelectedInfo elementInfo) {
    var singleSelected = elementInfo.element as SingleSelected;
    var text = elementInfo.selectedItem;
    if (text.length > 23) {
      text = text.substring(0, 22);
    }
    var widget = pw.Text(text,
        style: pw.TextStyle(
            font: ResourcesCache.pdfCustomFonts["defaultCN"],
            color: ReportStyleHelper.getPDFColorFromRTColor(
                singleSelected.fontColor!),
            fontSize: singleSelected.fontSize));

    return widget;
  }

  ///将MulitiSelectedInfo转换为具有PDF的文本组件的文本行
  static pw.Widget getMultiSelectPdfWidget(MulitiSelectedInfo elementInfo) {
    var mulitiSelected = elementInfo.element as MultiSelected;
    var text = _getStringList(elementInfo.selectedItems) ??
        mulitiSelected.items?.firstWhere((element) => element.isNotEmpty);

    if (text!.length > 23) {
      text = text.substring(0, 22);
    }
    var widget = pw.Text(text,
        style: pw.TextStyle(
            font: ResourcesCache.pdfCustomFonts["defaultCN"],
            color: ReportStyleHelper.getPDFColorFromRTColor(
                mulitiSelected.fontColor!),
            fontSize: mulitiSelected.fontSize));

    return widget;
  }

  ///将StaticImageInfo转换为具有PDF的文本组件图片
  ///数据源为Base64的图片数据
  static pw.Widget getStaticImagePdfWidget(StaticImageInfo elementInfo) {
    var image = elementInfo.element as StaticImage;

    Uint8List bytes = const Base64Decoder().convert(elementInfo.source!);
    // bytes = ReduceImageQualityAndSize(bytes);
    var memoryImage = pw.MemoryImage(bytes);

    var hor = image.horizontalAlignment;
    var ver = image.verticalAlignment;

    var imageHeight = image.imageHeight >= 784.0 ? 784.0 : image.imageHeight;
    return pw.Container(
        margin: const pw.EdgeInsets.fromLTRB(0, 1, 0, 0),
        child: pw.Wrap(
            alignment: ReportStyleHelper.getWrapAlignment(hor!),
            crossAxisAlignment: ReportStyleHelper.getWrapCrossAlignment(ver!),
            children: [
              pw.Image(memoryImage,
                  width: image.imageWidth, height: imageHeight)
            ]));
  }

  //返回带有容器和装饰器的图片Pdf组件
  static pw.Widget getInputImagePdfWidget(InputImageInfo elementInfo) {
    var image = elementInfo.element as InputImage;
    var isTransparent = image.transparentBackground;
    var hasBorder = image.hasImageBorder;

    pw.Border? borderSide = hasBorder!
        ? const pw.Border(
            left: pw.BorderSide(),
            right: pw.BorderSide(),
            top: pw.BorderSide(),
            bottom: pw.BorderSide())
        : null;
    PdfColor? pdfColor = isTransparent! ? null : PdfColors.black;
    var boxDecoration = pw.BoxDecoration(border: borderSide, color: pdfColor);

    pw.MemoryImage? meoryImage =
        ResourcesCache.getCacheCloudImages(elementInfo.selectedImage);
    return pw.Container(
        width: image.imageWidth,
        height: image.imageHeight,
        decoration: boxDecoration,
        alignment: pw.Alignment.bottomRight,
        child: meoryImage != null
            ? pw.Center(
                child: pw.Image(
                meoryImage,
                width: image.imageWidth,
                height: image.imageHeight,
              ))
            : pw.Center());
  }

  static String? _getStringList(List<String> strs) {
    var str = "";
    for (var element in strs) {
      str = str + element;
    }
    return str;
  }

  static _getCleanText(String rawText) {
    var text = rawText;
    var hasNewLine = text.contains("\r\n");
    text = hasNewLine ? text.replaceAll("\r", "") : text;
    return text;
  }
}
